@@ -8,7 +8,10 @@
 #include "nccl_net.h"
 #include <stdlib.h>
 #include <stdarg.h>
+#include <string.h>
+#include <strings.h>
 #include <sys/syscall.h>
+#include <chrono>
 #include "param.h"
 
 int ncclDebugLevel = -1;
@@ -16,14 +19,15 @@ static int pid = -1;
 static char hostname[1024];
 thread_local int ncclDebugNoWarn = 0;
 char ncclLastError[1024] = ""; // Global string for the last error in human readable form
-uint64_t ncclDebugMask = NCCL_INIT|NCCL_ENV; // Default debug sub-system mask is INIT and ENV
+static uint64_t ncclDebugMask = NCCL_INIT|NCCL_ENV; // Default debug sub-system mask is INIT and ENV
 FILE *ncclDebugFile = stdout;
-pthread_mutex_t ncclDebugLock = PTHREAD_MUTEX_INITIALIZER;
-std::chrono::steady_clock::time_point ncclEpoch;
+static pthread_mutex_t ncclDebugLock = PTHREAD_MUTEX_INITIALIZER;
+static std::chrono::steady_clock::time_point ncclEpoch;
+static bool ncclWarnSetDebugInfo = false;
 
 static __thread int tid = -1;
 
-void ncclDebugInit() {
+static void ncclDebugInit() {
   pthread_mutex_lock(&ncclDebugLock);
   if (ncclDebugLevel != -1) { pthread_mutex_unlock(&ncclDebugLock); return; }
   const char* nccl_debug = ncclGetEnv("NCCL_DEBUG");
@@ -83,6 +87,10 @@ void ncclDebugInit() {
         mask = NCCL_BOOTSTRAP;
       } else if (strcasecmp(subsys, "REG") == 0) {
         mask = NCCL_REG;
+      } else if (strcasecmp(subsys, "PROFILE") == 0) {
+        mask = NCCL_PROFILE;
+      } else if (strcasecmp(subsys, "VERBS") == 0) {
+        mask = NCCL_VERBS;
       } else if (strcasecmp(subsys, "ALL") == 0) {
         mask = NCCL_ALL;
       }
@@ -94,6 +102,15 @@ void ncclDebugInit() {
     free(ncclDebugSubsys);
   }
 
+  const char* ncclWarnSetDebugInfoEnv = ncclGetEnv("NCCL_WARN_ENABLE_DEBUG_INFO");
+  if (ncclWarnSetDebugInfoEnv != NULL && strlen(ncclWarnSetDebugInfoEnv) > 0) {
+    int64_t value;
+    errno = 0;
+    value = strtoll(ncclWarnSetDebugInfoEnv, NULL, 0);
+    if (!errno)
+      ncclWarnSetDebugInfo = value;
+  }
+
   // Cache pid and hostname
   getHostName(hostname, 1024, '.');
   pid = getpid();
@@ -143,8 +160,6 @@ void ncclDebugInit() {
   pthread_mutex_unlock(&ncclDebugLock);
 }
 
-NCCL_PARAM(WarnSetDebugInfo, "WARN_ENABLE_DEBUG_INFO", 0);
-
 /* Common logging function used by the INFO, WARN and TRACE macros
  * Also exported to the dynamically loadable Net transport modules so
  * they can share the debugging mechanisms and output files
@@ -178,7 +193,7 @@ void ncclDebugLog(ncclDebugLogLevel level, unsigned long flags, const char *file
   if (level == NCCL_LOG_WARN) {
     len = snprintf(buffer, sizeof(buffer), "\n%s:%d:%d [%d] %s:%d NCCL WARN ",
                    hostname, pid, tid, cudaDev, filefunc, line);
-    if (ncclParamWarnSetDebugInfo()) ncclDebugLevel = NCCL_LOG_INFO;
+    if (ncclWarnSetDebugInfo) ncclDebugLevel = NCCL_LOG_INFO;
   } else if (level == NCCL_LOG_INFO) {
     len = snprintf(buffer, sizeof(buffer), "%s:%d:%d [%d] NCCL INFO ", hostname, pid, tid, cudaDev);
   } else if (level == NCCL_LOG_TRACE && flags == NCCL_CALL) {
@@ -190,17 +205,15 @@ void ncclDebugLog(ncclDebugLogLevel level, unsigned long flags, const char *file
                    hostname, pid, tid, cudaDev, timestamp, filefunc, line);
   }
 
-  if (len) {
-    va_list vargs;
-    va_start(vargs, fmt);
-    len += vsnprintf(buffer+len, sizeof(buffer)-len, fmt, vargs);
-    va_end(vargs);
-    // vsnprintf may return len > sizeof(buffer) in the case of a truncated output.
-    // Rewind len so that we can replace the final \0 by \n
-    if (len > sizeof(buffer)) len = sizeof(buffer)-1;
-    buffer[len++] = '\n';
-    fwrite(buffer, 1, len, ncclDebugFile);
-  }
+  va_list vargs;
+  va_start(vargs, fmt);
+  len += vsnprintf(buffer+len, sizeof(buffer)-len, fmt, vargs);
+  va_end(vargs);
+  // vsnprintf may return len > sizeof(buffer) in the case of a truncated output.
+  // Rewind len so that we can replace the final \0 by \n
+  if (len > sizeof(buffer)) len = sizeof(buffer)-1;
+  buffer[len++] = '\n';
+  if (len) fwrite(buffer, 1, len, ncclDebugFile);
 }
 
 NCCL_PARAM(SetThreadName, "SET_THREAD_NAME", 0);
