@@ -10,6 +10,8 @@
 #define NCCL_COLLECTIVES_H_
 
 #include "nccl.h"
+#include "nccl_common.h"
+#include "device.h"
 
 // CHUNKSIZE must be a multiple of SLICESIZE
 #define ALLREDUCE_SLICESTEPS (NCCL_STEPS/4)
@@ -26,6 +28,12 @@
 #define ALLTOALL_PIVOT_SLICESTEPS 2
 #define ALLTOALL_PIVOT_CHUNKSTEPS 4
 
+const char* ncclFuncToString(ncclFunc_t op);
+const char* ncclDevRedOpToString(ncclDevRedOp_t op);
+const char* ncclDatatypeToString(ncclDataType_t type);
+const char* ncclAlgoToString(int algo);
+const char* ncclProtoToString(int proto);
+
 inline int ncclTypeSize(ncclDataType_t type) {
   switch (type) {
   case ncclInt8:
