@@ -11,11 +11,11 @@
 #include "comm.h"
 #include "msccl/msccl_struct.h"
 
-ncclResult_t mscclGetCaptureStatus(hipStream_t stream);
+ncclResult_t mscclGetCaptureStatus(int rank, hipStream_t stream);
 
 ncclResult_t mscclSetupScratch(struct mscclAlgo* hostAlgo, hipStream_t stream);
 
-ncclResult_t mscclSetupSyncFlags(hipStream_t stream);
+ncclResult_t mscclSetupSyncFlags(int rank, hipStream_t stream);
 
 ncclResult_t mscclSetupConnections(struct mscclAlgo* hostAlgo, ncclComm_t comm);
 
