@@ -573,7 +573,7 @@ ncclResult_t mscclGetAlgoFromXmlFile(const char* str, struct mscclAlgo* algo, in
                   mscclTran->srcOffset = srcOffset;
                   mscclTran->dstBuffer = dstBufferInt;
                   mscclTran->dstOffset = dstOffset;
-
+                  algo->typeMask |= (1<<transferType);
                   if (count < 0 || count >= MSCCL_MAX_COUNT){
                     WARN("MSCCL: count (%d) must be positive and less than %d", count, MSCCL_MAX_COUNT);
                     return ncclInternalError;
@@ -658,7 +658,7 @@ ncclResult_t mscclGetAlgoFromXmlFile(const char* str, struct mscclAlgo* algo, in
 
             // finish up mscclChannel calculation
 
-            for (int c = 0; c < MSCCL_MAX_COUNT; c++) {
+            for (int c = 1; c <= MSCCL_MAX_COUNT; c++) {
               struct mscclChannelPeerInfo* sendPeer = &mscclChannel->sendPeerInfo[mscclChannel->nSendPeers];
               if (sendPeer->nTransmissionsOfCount[c] > 0) {
                 sendPeer->existingCounts[sendPeer->nExistingCounts] = c;
@@ -731,6 +731,10 @@ ncclResult_t mscclGetAlgoMetaFromXmlFile(const char* str, struct mscclAlgoMeta*
   NCCLCHECK(mscclXmlGetAttrInt(node, "nchunksperloop", &nChunksPerLoop));
   algoMeta->nChunksPerLoop  = nChunksPerLoop;
 
+  int nChannels;
+  NCCLCHECK(mscclXmlGetAttrInt(node, "nchannels", &nChannels));
+  algoMeta->nChannels = nChannels;
+
   int nGpus;
   NCCLCHECK(mscclXmlGetAttrInt(node, "ngpus", &nGpus));
   algoMeta->nRanks = nGpus;
