@@ -41,6 +41,7 @@ namespace RcclUnitTesting
 
     std::vector<int>            const& GetNumGpusList();
     std::vector<int>            const& GetIsMultiProcessList();
+    std::vector<int>            const& GetGpuPriorityOrder();   // Orders the gpus based on the associativity of them with OAM with higher gpus linked.
     void ShowConfig();
 
   protected:
@@ -49,6 +50,7 @@ namespace RcclUnitTesting
     std::vector<int>            numGpusList;        // List of # Gpus to use   [UT_MIN_GPUS/UT_MAX_GPUS/UT_POW2_GPUS]
     std::vector<int>            isMultiProcessList; // Single or multi process [UT_PROCESS_MASK]
     int                         numDetectedGpus;
+    std::vector<int>            gpuPriorityOrder;   // Orders the gpus based on the associativity of them with OAM with higher gpus linked.
 
     // Helper functions to parse environment variables
     int GetEnvVar(std::string const varname, int defaultValue);
